@@ -16,16 +16,24 @@
  */
 package org.apache.activemq.artemis.tests.integration.management;
 
-import javax.management.openmbean.CompositeData;
 import java.util.HashMap;
 import java.util.Map;
 
+import javax.management.openmbean.CompositeData;
+
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.management.QueueControl;
 import org.apache.activemq.artemis.api.core.management.ResourceNames;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
 
+@RunWith(value = Parameterized.class)
 public class QueueControlUsingCoreTest extends QueueControlTest {
 
+   public QueueControlUsingCoreTest(boolean durable) {
+      super(durable);
+   }
+
    @Override
    protected QueueControl createManagementControl(final SimpleString address,
                                                   final SimpleString queue) throws Exception {
@@ -41,6 +49,99 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             }
          }
 
+         @Override
+         public int getPreparedTransactionMessageCount() {
+            try {
+               return (Integer) proxy.invokeOperation(Integer.class, "getPreparedTransactionMessageCount");
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public void deliverScheduledMessages(String filter) throws Exception {
+            try {
+               proxy.invokeOperation("deliverScheduledMessages", filter);
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public void deliverScheduledMessage(long messageId) throws Exception {
+            try {
+               proxy.invokeOperation("deliverScheduledMessage", messageId);
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public boolean isAutoDelete() {
+            return (Boolean) proxy.retrieveAttributeValue("autoDelete");
+         }
+
+         @Override
+         public void resetAllGroups() {
+            try {
+               proxy.invokeOperation("resetAllGroups");
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public void resetGroup(String groupID) {
+            try {
+               proxy.invokeOperation("resetGroup", groupID);
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public int getGroupCount() {
+            try {
+               return (Integer) proxy.invokeOperation(Integer.TYPE, "getGroupCount");
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public String listGroupsAsJSON() throws Exception {
+            return (String) proxy.invokeOperation("listGroupsAsJSON");
+         }
+
+         @Override
+         public long getRingSize() {
+            try {
+               return (Long) proxy.invokeOperation(Integer.TYPE, "getRingSize");
+            } catch (Exception e) {
+               throw new RuntimeException(e.getMessage(), e);
+            }
+         }
+
+         @Override
+         public boolean isGroupRebalance() {
+            return (Boolean) proxy.retrieveAttributeValue("groupRebalance");
+         }
+
+         @Override
+         public boolean isGroupRebalancePauseDispatch() {
+            return (Boolean) proxy.retrieveAttributeValue("groupRebalancePauseDispatch");
+         }
+
+         @Override
+         public int getGroupBuckets() {
+            return (Integer) proxy.retrieveAttributeValue("groupBuckets", Integer.class);
+         }
+
+         @Override
+         public String getGroupFirstKey() {
+            return (String) proxy.retrieveAttributeValue("groupFirstKey");
+         }
+
          @Override
          public boolean changeMessagePriority(final long messageID, final int newPriority) throws Exception {
             return (Boolean) proxy.invokeOperation("changeMessagePriority", messageID, newPriority);
@@ -56,6 +157,26 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (Long) proxy.invokeOperation(Long.class, "countMessages", filter);
          }
 
+         @Override
+         public long countMessages() throws Exception {
+            return (Long) proxy.invokeOperation(Long.class, "countMessages");
+         }
+
+         @Override
+         public String countMessages(final String filter, final String groupByFilter) throws Exception {
+            return (String) proxy.invokeOperation(String.class, "countMessages", filter, groupByFilter);
+         }
+
+         @Override
+         public long countDeliveringMessages(final String filter) throws Exception {
+            return (Long) proxy.invokeOperation(Long.class, "countDeliveringMessages", filter);
+         }
+
+         @Override
+         public String countDeliveringMessages(final String filter, final String groupByFilter) throws Exception {
+            return (String) proxy.invokeOperation(String.class, "countDeliveringMessages", filter, groupByFilter);
+         }
+
          @Override
          public boolean expireMessage(final long messageID) throws Exception {
             return (Boolean) proxy.invokeOperation("expireMessage", messageID);
@@ -71,6 +192,11 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (String) proxy.retrieveAttributeValue("address");
          }
 
+         @Override
+         public String getUser() {
+            return (String) proxy.retrieveAttributeValue("user");
+         }
+
          @Override
          public int getConsumerCount() {
             return (Integer) proxy.retrieveAttributeValue("consumerCount", Integer.class);
@@ -91,11 +217,71 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (Boolean) proxy.retrieveAttributeValue("purgeOnNoConsumers");
          }
 
+         @Override
+         public boolean isEnabled() {
+            return  (Boolean) proxy.retrieveAttributeValue("isEnabled");
+         }
+
+         @Override
+         public void enable() throws Exception {
+            proxy.invokeOperation("enable");
+         }
+
+         @Override
+         public void disable() throws Exception {
+            proxy.invokeOperation("disable");
+         }
+
+         @Override
+         public boolean isConfigurationManaged() {
+            return (Boolean) proxy.retrieveAttributeValue("configurationManaged");
+         }
+
+         @Override
+         public boolean isExclusive() {
+            return (Boolean) proxy.retrieveAttributeValue("exclusive");
+         }
+
+         @Override
+         public boolean isLastValue() {
+            return (Boolean) proxy.retrieveAttributeValue("lastValue");
+         }
+
+         @Override
+         public String getLastValueKey() {
+            return (String) proxy.retrieveAttributeValue("lastValueKey");
+         }
+
+         @Override
+         public int getConsumersBeforeDispatch() {
+            return (Integer) proxy.retrieveAttributeValue("consumersBeforeDispatch");
+         }
+
+         @Override
+         public long getDelayBeforeDispatch() {
+            return (Long) proxy.retrieveAttributeValue("delayBeforeDispatch");
+         }
+
          @Override
          public int getDeliveringCount() {
             return (Integer) proxy.retrieveAttributeValue("deliveringCount", Integer.class);
          }
 
+         @Override
+         public long getDeliveringSize() {
+            return (Long) proxy.retrieveAttributeValue("deliveringSize", Long.class);
+         }
+
+         @Override
+         public int getDurableDeliveringCount() {
+            return (Integer) proxy.retrieveAttributeValue("durableDeliveringCount", Integer.class);
+         }
+
+         @Override
+         public long getDurableDeliveringSize() {
+            return (Long) proxy.retrieveAttributeValue("durableDeliveringSize", Long.class);
+         }
+
          @Override
          public String getExpiryAddress() {
             return (String) proxy.retrieveAttributeValue("expiryAddress");
@@ -121,6 +307,11 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (Integer) proxy.retrieveAttributeValue("messagesAcknowledged", Integer.class);
          }
 
+         @Override
+         public long getAcknowledgeAttempts() {
+            return (Integer) proxy.retrieveAttributeValue("acknowledgeAttempts", Integer.class);
+         }
+
          @Override
          public long getMessagesExpired() {
             return (Long) proxy.retrieveAttributeValue("messagesExpired", Long.class);
@@ -166,16 +357,41 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (Long) proxy.retrieveAttributeValue("scheduledCount", Long.class);
          }
 
+         @Override
+         public long getScheduledSize() {
+            return (Long) proxy.retrieveAttributeValue("scheduledSize", Long.class);
+         }
+
+         @Override
+         public long getDurableScheduledCount() {
+            return (Long) proxy.retrieveAttributeValue("durableScheduledCount", Long.class);
+         }
+
+         @Override
+         public long getDurableScheduledSize() {
+            return (Long) proxy.retrieveAttributeValue("durableScheduledSize", Long.class);
+         }
+
          @Override
          public boolean isDurable() {
             return (Boolean) proxy.retrieveAttributeValue("durable");
          }
 
+         @Override
+         public String getRoutingType() {
+            return (String) proxy.retrieveAttributeValue("routingType");
+         }
+
          @Override
          public boolean isTemporary() {
             return (Boolean) proxy.retrieveAttributeValue("temporary");
          }
 
+         @Override
+         public boolean isRetroactiveResource() {
+            return (Boolean) proxy.retrieveAttributeValue("retroactiveResource");
+         }
+
          @Override
          public String listMessageCounter() throws Exception {
             return (String) proxy.invokeOperation("listMessageCounter");
@@ -263,6 +479,11 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (Integer) proxy.invokeOperation("moveMessages", flushLimit, filter, otherQueueName, rejectDuplicates);
          }
 
+         @Override
+         public int moveMessages(int flushLimit, String filter, String otherQueueName, boolean rejectDuplicates, int messageCount) throws Exception {
+            return (Integer) proxy.invokeOperation(Integer.class, "moveMessages", flushLimit, filter, otherQueueName, rejectDuplicates, messageCount);
+         }
+
          @Override
          public int moveMessages(final String filter,
                                  final String otherQueueName,
@@ -302,6 +523,11 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (Integer) proxy.invokeOperation(Integer.class, "removeMessages", limit, filter);
          }
 
+         @Override
+         public int removeAllMessages() throws Exception {
+            return (Integer) proxy.invokeOperation(Integer.class, "removeAllMessages");
+         }
+
          @Override
          public boolean removeMessage(final long messageID) throws Exception {
             return (Boolean) proxy.invokeOperation("removeMessage", messageID);
@@ -332,6 +558,17 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return (String) proxy.invokeOperation("sendMessage", headers, type, body, durable, user, password);
          }
 
+         @Override
+         public String sendMessage(Map<String, String> headers,
+                                   int type,
+                                   String body,
+                                   boolean durable,
+                                   String user,
+                                   String password,
+                                   boolean createMessageId) throws Exception {
+            return (String) proxy.invokeOperation("sendMessage", headers, type, body, durable, user, password, createMessageId);
+         }
+
          public void setDeadLetterAddress(final String deadLetterAddress) throws Exception {
             proxy.invokeOperation("setDeadLetterAddress", deadLetterAddress);
          }
@@ -370,6 +607,25 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
             return compositeDatas;
          }
 
+         @Override
+         public CompositeData[] browse(int page, int pageSize) throws Exception {
+            Map map = (Map) proxy.invokeOperation("browse", page, pageSize);
+            CompositeData[] compositeDatas = (CompositeData[]) map.get(CompositeData.class.getName());
+            if (compositeDatas == null) {
+               compositeDatas = new CompositeData[0];
+            }
+            return compositeDatas;
+         }
+
+         @Override
+         public CompositeData[] browse(int page, int pageSize, String filter) throws Exception {
+            Map map = (Map) proxy.invokeOperation("browse", page, pageSize, filter);
+            CompositeData[] compositeDatas = (CompositeData[]) map.get(CompositeData.class.getName());
+            if (compositeDatas == null) {
+               compositeDatas = new CompositeData[0];
+            }
+            return compositeDatas;
+         }
 
          @Override
          public CompositeData[] browse(String filter) throws Exception {
@@ -415,6 +671,21 @@ public class QueueControlUsingCoreTest extends QueueControlTest {
          public String listDeliveringMessagesAsJSON() throws Exception {
             return (String) proxy.invokeOperation("listDeliveringMessagesAsJSON");
          }
+
+         @Override
+         public long getPersistentSize() {
+            return (Long) proxy.retrieveAttributeValue("persistentSize", Long.class);
+         }
+
+         @Override
+         public long getDurableMessageCount() {
+            return (Long) proxy.retrieveAttributeValue("durableMessageCount", Long.class);
+         }
+
+         @Override
+         public long getDurablePersistentSize() {
+            return (Long) proxy.retrieveAttributeValue("durablePersistentSize", Long.class);
+         }
       };
    }
 }
